@@ -1,8 +1,10 @@
+import logging
 import spacy
 import requests
 from spacy.language import Language
 from spacy.tokens import Span, Doc
 
+log = logging.getLogger(__name__)
 
 @Language.factory('opentapioca',
                   default_config={"url": "https://opentapioca.org/api/annotate"})
@@ -20,6 +22,10 @@ class EntityLinker(object):
         Span.set_extension("rank", default=None, force=True)
         Span.set_extension("score", default=None, force=True)
         Span.set_extension("types", default=None, force=True)
+        Span.set_extension("label", default=None, force=True)
+        Span.set_extension("extra_aliases", default=None, force=True)
+        Span.set_extension("nb_sitelinks", default=None, force=True)
+        Span.set_extension("nb_statements", default=None, force=True)
 
     def __call__(self, doc):
         """Requests the OpenTapioca API. Attaches entities to spans and doc."""
@@ -50,17 +56,28 @@ class EntityLinker(object):
                              'LOC': t['Q618123'] + t['P1566']}
                     m = max(types.values())
                     etype = ''.join([k for k, v in types.items() if v == m])
-                except Exception:
+                except Exception as e:
+                    log.error(e, extra=ent)
                     etype = ''
                 span = doc.char_span(start, end, etype, ent_kb_id)
             else:
-                span = doc.char_span(start, end, '')
+                etype, ent_kb_id = '', ''
+                span = doc.char_span(start, end, etype)
+            if not span:
+                span = doc.char_span(start, end, etype, ent_kb_id,
+                                     alignment_mode='expand')
+                log.warning('The OpenTapioca-entity "%s" %s does not fit the span "%s" %s in spaCy. EXPANDED!',
+                            ent['tags'][0]['label'][0], (start,end), span.text, (span.start_char, span.end_char))
             span._.annotations = ent
             span._.description = ent['tags'][0]['desc']
             span._.aliases = ent['tags'][0]['aliases']
             span._.rank = ent['tags'][0]['rank']
             span._.score = ent['tags'][0]['score']
             span._.types = ent['tags'][0]['types']
+            span._.label = ent['tags'][0]['label']
+            span._.extra_aliases = ent['tags'][0]['extra_aliases']
+            span._.nb_sitelinks = ent['tags'][0]['nb_sitelinks']
+            span._.nb_statements = ent['tags'][0]['nb_statements']
             ents.append(span)
 
         # Attach processed entities to doc.ents
@@ -68,8 +85,8 @@ class EntityLinker(object):
             # this works with non-overlapping spans
             doc.ents = list(doc.ents) + ents
         except Exception:
-            # filter the  overlapping spans, keep the (first) longest one
-            doc.ents = list(doc.ents) + spacy.util.filter_spans(ents)
+            # filter the overlapping spans, keep the (first) longest one
+            doc.ents = spacy.util.filter_spans(ents)
         # Attach all entities found by OpenTapioca to spans
         doc.spans['all_entities_opentapioca'] = ents
         return doc
